@@ -210,7 +210,7 @@ uint32_t FAudio_Initialize(
 	FAudioProcessor XAudio2Processor
 ) {
 	LOG_API_ENTER(audio)
-	FAudio_assert(Flags == 0 || Flags == FAUDIO_DEBUG_ENGINE);
+	FAudio_assert((Flags & ~(FAUDIO_DEBUG_ENGINE | FAUDIO_1024_QUANTUM)) == 0);
 	FAudio_assert(XAudio2Processor == FAUDIO_DEFAULT_PROCESSOR);
 
 	audio->initFlags = Flags;
@@ -2265,11 +2265,71 @@ void FAudioVoice_GetOutputMatrix(
 	LOG_API_EXIT(voice->audio)
 }
 
-void FAudioVoice_DestroyVoice(FAudioVoice *voice)
+static uint32_t check_for_sends_to_voice(FAudioVoice *voice)
 {
+	FAudio *audio = voice->audio;
+	uint32_t ret = 0;
+	FAudioSourceVoice *source;
+	FAudioSubmixVoice *submix;
+	LinkedList *list;
 	uint32_t i;
+
+	FAudio_PlatformLockMutex(audio->sourceLock);
+	list = audio->sources;
+	while (list != NULL)
+	{
+		source = (FAudioSourceVoice*) list->entry;
+		for (i = 0; i < source->sends.SendCount; i += 1)
+			if (source->sends.pSends[i].pOutputVoice == voice)
+			{
+				ret = 0x80004005; /* E_FAIL */
+				break;
+			}
+		if (ret)
+			break;
+		list = list->next;
+	}
+	FAudio_PlatformUnlockMutex(audio->sourceLock);
+
+	if (ret)
+		return ret;
+
+	FAudio_PlatformLockMutex(audio->submixLock);
+	list = audio->submixes;
+	while (list != NULL)
+	{
+		submix = (FAudioSubmixVoice*) list->entry;
+		for (i = 0; i < submix->sends.SendCount; i += 1)
+			if (submix->sends.pSends[i].pOutputVoice == voice)
+			{
+				ret = 0x80004005; /* E_FAIL */
+				break;
+			}
+		if (ret)
+			break;
+		list = list->next;
+	}
+	FAudio_PlatformUnlockMutex(audio->submixLock);
+
+	return ret;
+}
+
+uint32_t FAudioVoice_DestroyVoiceSafeEXT(FAudioVoice *voice)
+{
+	uint32_t i, ret;
 	LOG_API_ENTER(voice->audio)
 
+	if ((ret = check_for_sends_to_voice(voice)))
+	{
+		LOG_ERROR(
+			voice->audio,
+			"Voice %p is an output for other voice(s)",
+			voice
+		)
+		LOG_API_EXIT(voice->audio)
+		return ret;
+	}
+
 	/* TODO: Check for dependencies and remove from audio graph first! */
 	FAudio_OPERATIONSET_ClearAllForVoice(voice);
 
@@ -2443,6 +2503,12 @@ void FAudioVoice_DestroyVoice(FAudioVoice *voice)
 	LOG_API_EXIT(voice->audio)
 	FAudio_Release(voice->audio);
 	voice->audio->pFree(voice);
+	return 0;
+}
+
+void FAudioVoice_DestroyVoice(FAudioVoice *voice)
+{
+	FAudioVoice_DestroyVoiceSafeEXT(voice);
 }
 
 /* FAudioSourceVoice Interface */
@@ -2512,7 +2578,7 @@ uint32_t FAudioSourceVoice_SubmitSourceBuffer(
 	const FAudioBufferWMA *pBufferWMA
 ) {
 	uint32_t adpcmMask, *adpcmByteCount;
-	uint32_t playBegin, playLength, loopBegin, loopLength;
+	uint32_t playBegin, playLength, loopBegin, loopLength, bufferLength;
 	FAudioBufferEntry *entry, *list;
 
 	LOG_API_ENTER(voice->audio)
@@ -2551,37 +2617,43 @@ uint32_t FAudioSourceVoice_SubmitSourceBuffer(
 		return FAUDIO_E_INVALID_CALL;
 	}
 
+	if (voice->src.format->wFormatTag == FAUDIO_FORMAT_MSADPCM)
+	{
+		FAudioADPCMWaveFormat *fmtex = (FAudioADPCMWaveFormat*) voice->src.format;
+		bufferLength =
+			pBuffer->AudioBytes /
+			fmtex->wfx.nBlockAlign *
+			fmtex->wSamplesPerBlock;
+	}
+	else if (voice->src.format->wFormatTag == FAUDIO_FORMAT_XMAUDIO2)
+	{
+		FAudioXMA2WaveFormat *fmtex = (FAudioXMA2WaveFormat*) voice->src.format;
+		bufferLength = fmtex->dwSamplesEncoded;
+	}
+	else if (pBufferWMA != NULL)
+	{
+		bufferLength =
+			pBufferWMA->pDecodedPacketCumulativeBytes[pBufferWMA->PacketCount - 1] /
+			(voice->src.format->nChannels * voice->src.format->wBitsPerSample / 8);
+	}
+	else
+	{
+		bufferLength =
+			pBuffer->AudioBytes /
+			voice->src.format->nBlockAlign;
+	}
+
 	/* PlayLength Default */
 	if (playLength == 0)
 	{
-		if (voice->src.format->wFormatTag == FAUDIO_FORMAT_MSADPCM)
-		{
-			FAudioADPCMWaveFormat *fmtex = (FAudioADPCMWaveFormat*) voice->src.format;
-			playLength = (
-				pBuffer->AudioBytes /
-				fmtex->wfx.nBlockAlign *
-				fmtex->wSamplesPerBlock
-			) - playBegin;
-		}
-		else if (voice->src.format->wFormatTag == FAUDIO_FORMAT_XMAUDIO2)
-		{
-			FAudioXMA2WaveFormat *fmtex = (FAudioXMA2WaveFormat*) voice->src.format;
-			playLength = fmtex->dwSamplesEncoded - playBegin;
-		}
-		else if (pBufferWMA != NULL)
-		{
-			playLength = (
-				pBufferWMA->pDecodedPacketCumulativeBytes[pBufferWMA->PacketCount - 1] /
-				(voice->src.format->nChannels * voice->src.format->wBitsPerSample / 8)
-			) - playBegin;
-		}
-		else
-		{
-			playLength = (
-				pBuffer->AudioBytes /
-				voice->src.format->nBlockAlign
-			) - playBegin;
-		}
+		playLength = bufferLength - playBegin;
+	}
+	else if (playBegin + playLength > bufferLength || playBegin + playLength < playLength)
+	{
+		/* Reading past the end of the buffer, or begin + length overflow uint32_t, which
+		 * would also read past the end of the buffer. */
+		LOG_API_EXIT(voice->audio)
+		return FAUDIO_E_INVALID_CALL;
 	}
 
 	if (pBuffer->LoopCount > 0 && pBufferWMA == NULL && voice->src.format->wFormatTag != FAUDIO_FORMAT_XMAUDIO2)
