@@ -1,6 +1,6 @@
 /* FAudio - XAudio Reimplementation for FNA
  *
- * Copyright (c) 2011-2023 Ethan Lee, Luigi Auriemma, and the MonoGame Team
+ * Copyright (c) 2011-2024 Ethan Lee, Luigi Auriemma, and the MonoGame Team
  *
  * This software is provided 'as-is', without any express or implied warranty.
  * In no event will the authors be held liable for any damages arising from
@@ -24,7 +24,7 @@
  *
  */
 
-#ifndef FAUDIO_WIN32_PLATFORM
+#if !defined(FAUDIO_WIN32_PLATFORM) && !defined(FAUDIO_SDL3_PLATFORM)
 
 #include "FAudio_internal.h"
 
@@ -55,6 +55,7 @@ static void FAudio_INTERNAL_MixCallback(void *userdata, Uint8 *stream, int len)
 static void FAudio_INTERNAL_PrioritizeDirectSound()
 {
 	int numdrivers, i, wasapi, directsound;
+	void *dll, *proc;
 
 	if (SDL_GetHint("SDL_AUDIODRIVER") != NULL)
 	{
@@ -62,6 +63,28 @@ static void FAudio_INTERNAL_PrioritizeDirectSound()
 		return;
 	}
 
+	/* Windows 10+ decided to break version detection, so instead of doing
+	 * it the right way we have to do something dumb like search for an
+	 * export that's only in Windows 10 or newer.
+	 * -flibit
+	 */
+	if (SDL_strcmp(SDL_GetPlatform(), "Windows") != 0)
+	{
+		return;
+	}
+	dll = SDL_LoadObject("USER32.DLL");
+	if (dll == NULL)
+	{
+		return;
+	}
+	proc = SDL_LoadFunction(dll, "SetProcessDpiAwarenessContext");
+	SDL_UnloadObject(dll); /* We aren't really using this, unload now */
+	if (proc != NULL)
+	{
+		/* OS is new enough to trust WASAPI, bail */
+		return;
+	}
+
 	/* Check to see if we have both Windows drivers in the list */
 	numdrivers = SDL_GetNumAudioDrivers();
 	wasapi = -1;
@@ -123,11 +146,12 @@ void FAudio_PlatformInit(
 
 	FAudio_assert(mixFormat != NULL);
 	FAudio_assert(updateSize != NULL);
+	FAudio_assert((mixFormat->Format.nChannels <= 255) && "mixFormat->Format.nChannels out of range!");
 
 	/* Build the device spec */
 	want.freq = mixFormat->Format.nSamplesPerSec;
-	want.format = AUDIO_F32;
-	want.channels = mixFormat->Format.nChannels;
+	want.format = AUDIO_F32SYS;
+	want.channels = (Uint8)(mixFormat->Format.nChannels);
 	want.silence = 0;
 	want.callback = FAudio_INTERNAL_MixCallback;
 	want.userdata = audio;
@@ -260,7 +284,7 @@ uint32_t FAudio_PlatformGetDeviceDetails(
 	);
 
 	/* Environment variables take precedence over all possible values */
-	envvar = SDL_getenv("SDL_AUDIO_FREQUENCY");
+	envvar = SDL_GetHint("SDL_AUDIO_FREQUENCY");
 	if (envvar != NULL)
 	{
 		rate = SDL_atoi(envvar);
@@ -269,7 +293,7 @@ uint32_t FAudio_PlatformGetDeviceDetails(
 	{
 		rate = 0;
 	}
-	envvar = SDL_getenv("SDL_AUDIO_CHANNELS");
+	envvar = SDL_GetHint("SDL_AUDIO_CHANNELS");
 	if (envvar != NULL)
 	{
 		channels = SDL_atoi(envvar);
@@ -296,7 +320,7 @@ uint32_t FAudio_PlatformGetDeviceDetails(
 	{
 		rate = spec.freq;
 	}
-	if ((spec.channels > 0) && (channels <= 0))
+	if ((spec.channels > 0) && (spec.channels < 9) && (channels <= 0))
 	{
 		channels = spec.channels;
 	}
@@ -648,4 +672,4 @@ void FAudio_UTF8_To_UTF16(const char *src, uint16_t *dst, size_t len)
 
 extern int this_tu_is_empty;
 
-#endif /* FAUDIO_WIN32_PLATFORM */
+#endif /* !defined(FAUDIO_WIN32_PLATFORM) && !defined(FAUDIO_SDL3_PLATFORM) */
