@@ -1,5 +1,7 @@
 #include <stdio.h>
-#include <SDL.h>
+#include <SDL3/SDL.h>
+#include <stdlib.h> /* malloc */
+#include <stdio.h> /* FILE* */
 
 #include <FAudio.h>
 
